#include <Dynamixel2Arduino.h>

// Please modify it to suit your hardware.
#if defined(ARDUINO_AVR_UNO) || defined(ARDUINO_AVR_MEGA2560) // When using DynamixelShield
  #include <SoftwareSerial.h>
  SoftwareSerial soft_serial(7, 8); // DYNAMIXELShield UART RX/TX
  #define DXL_SERIAL   Serial
  #define DEBUG_SERIAL soft_serial
  const int DXL_DIR_PIN = 2; // DYNAMIXEL Shield DIR PIN
#elif defined(ARDUINO_SAM_DUE) // When using DynamixelShield
  #define DXL_SERIAL   Serial
  #define DEBUG_SERIAL SerialUSB
  const int DXL_DIR_PIN = 2; // DYNAMIXEL Shield DIR PIN
#elif defined(ARDUINO_SAM_ZERO) // When using DynamixelShield
  #define DXL_SERIAL   Serial1
  #define DEBUG_SERIAL SerialUSB
  const int DXL_DIR_PIN = 2; // DYNAMIXEL Shield DIR PIN
#else // Other boards when using DynamixelShield
  #define DXL_SERIAL   Serial1
  #define DEBUG_SERIAL Serial
  const int DXL_DIR_PIN = 2; // DYNAMIXEL Shield DIR PIN
#endif
 
const uint8_t DXL_ID1 = 2; //모터아이디 지정
const uint8_t DXL_ID2 = 6;
const uint8_t DXL_ID3 = 18;
const uint8_t DXL_ID4 = 15;
const uint8_t DXL_ID5 = 8; 
const uint8_t DXL_ID6 = 1; 

const float DXL_PROTOCOL_VERSION = 1.0; // 통신 프로토콜 버전 


Dynamixel2Arduino dxl(DXL_SERIAL, DXL_DIR_PIN);

//This namespace is required to use Control table item names
using namespace ControlTableItem;

void setup() {
  // put your setup code here, to run once: 
  // Use UART port of DYNAMIXEL Shield to debug.
  DEBUG_SERIAL.begin(1000000);
  while(!DEBUG_SERIAL);

  // Set Port baudrate to 57600bps. This has to match with DYNAMIXEL baudrate.
  dxl.begin(1000000);
  // Set Port Protocol Version. This has to match with DYNAMIXEL protocol version.
  dxl.setPortProtocolVersion(DXL_PROTOCOL_VERSION);
  // Get DYNAMIXEL information
  dxl.ping(DXL_ID1);
  dxl.ping(DXL_ID2);
  dxl.ping(DXL_ID3);
  dxl.ping(DXL_ID4);
  dxl.ping(DXL_ID5);
  dxl.ping(DXL_ID6);

  // Turn off torque when configuring items in EEPROM area
  dxl.torqueOff(DXL_ID1);
  dxl.setOperatingMode(DXL_ID1, OP_POSITION);
  dxl.torqueOn(DXL_ID1);
  dxl.torqueOff(DXL_ID2);
  dxl.setOperatingMode(DXL_ID2, OP_POSITION);
  dxl.torqueOn(DXL_ID2);
  dxl.torqueOff(DXL_ID3);
  dxl.setOperatingMode(DXL_ID3, OP_POSITION);
  dxl.torqueOn(DXL_ID3);
  dxl.torqueOff(DXL_ID4);
  dxl.setOperatingMode(DXL_ID4, OP_POSITION);
  dxl.torqueOn(DXL_ID4);
  dxl.torqueOff(DXL_ID5);
  dxl.setOperatingMode(DXL_ID5, OP_POSITION);
  dxl.torqueOn(DXL_ID5);
  dxl.torqueOff(DXL_ID6);
  dxl.setOperatingMode(DXL_ID6, OP_POSITION);
  dxl.torqueOn(DXL_ID6);
  
  // Limit the maximum velocity in Position Control Mode. Use 0 for Max speed
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID1, 30);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID2, 30);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID3, 30);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID4, 30);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID5, 30);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID6, 30);
  
}

void loop() 
{
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID1, 100);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID2, 100);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID3, 100);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID4, 100);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID5, 100);
  dxl.writeControlTableItem(MOVING_SPEED, DXL_ID6, 100);
  for(int i=0 ; i<1 ; i++)
  {
    delay(10000);
    set_position();
    delay(1000);
    cocacola_a();
    delay(1000);
    middle_point_bottom();
    delay(1000);
    cocacola_b(); 
    delay(1000);
    end_point_bottom();   
  }
}

void ultrasonic()
{

}

void set_position()  // 초기자세 조정 
{
  dxl.setGoalPosition(DXL_ID1, 150.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID2, 60.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 60.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 150.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 150.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID6, 150.0, UNIT_DEGREE);
  delay(1000);
}

void middle_point_top() //물품보관함에서 물건 뺄때 중간과정(상단) 
{
  dxl.setGoalPosition(DXL_ID2, 60.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 60.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 202, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 202, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 40.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 40.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 210, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 210, UNIT_DEGREE);
  delay(5000);
}

void middle_point_bottom() //물품보관함에서 물건 뺄때 중간과정(하단)
{
  dxl.setGoalPosition(DXL_ID4, 270.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 270.0, UNIT_DEGREE);
  delay(1000);
  dxl.setGoalPosition(DXL_ID2, 35.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 35.0, UNIT_DEGREE);
  delay(3000);

}

void end_point_top() //쇼케이스에서 로봇팔 뺄때 중간과정(상단) 
{
  dxl.setGoalPosition(DXL_ID2, 45.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 45.0, UNIT_DEGREE);
  delay(3000);
}

void end_point_bottom()
{
  dxl.setGoalPosition(DXL_ID2, 40.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 40.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 299.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 299.0, UNIT_DEGREE);
  delay(3000);
}

// 여기서 부터 물품보관함으로 옮기는 함수 

void fanta_a()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 257.5, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 65.5, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 65.5, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID4, 220.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 220.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID6, 240.0, UNIT_DEGREE);
  delay(3000);
}

void pepsi_a()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 240, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 63.5, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 63.5, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID4, 221.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 221.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID6, 240.0, UNIT_DEGREE);
  delay(3000);
}

void two_percent_a()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 217.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 65.5, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 65.5, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID4, 220.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 220.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID6, 240.0, UNIT_DEGREE);
  delay(3000);
}

void cocacola_a()
{
  dxl.setGoalPosition(DXL_ID1, 256.5, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID2, 47.6, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 47.6, UNIT_DEGREE);
  delay(5000);
  dxl.setGoalPosition(DXL_ID4, 280.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 280.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 82.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 82.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID4, 255.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 255.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID6, 240, UNIT_DEGREE);
  delay(3000);
}

void chilsung_a()
{
  dxl.setGoalPosition(DXL_ID1, 240.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID2, 47.6, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 47.6, UNIT_DEGREE);
  delay(5000);
  dxl.setGoalPosition(DXL_ID4, 270.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 270.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 75.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 75.0, UNIT_DEGREE);
  delay(1500);
  dxl.setGoalPosition(DXL_ID4, 262.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 262.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID2, 76.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 76.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID6, 240, UNIT_DEGREE);
  delay(3000);
}

void tejava_a()
{
  dxl.setGoalPosition(DXL_ID1, 217.5, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID2, 47.6, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 47.6, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID4, 280.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 280.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID2, 82.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 82.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID4, 255.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 255.0, UNIT_DEGREE);
  delay(3000);
  dxl.setGoalPosition(DXL_ID6, 240, UNIT_DEGREE);
  delay(3000);
}

// 여기서 부터 쇼케이스로 옮기는 함수 

void fanta_b()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 47.5, UNIT_DEGREE);
  delay(10000);
  dxl.setGoalPosition(DXL_ID2, 65.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 65.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 218.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 218.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID6, 150, UNIT_DEGREE);
  delay(3000);
}

void pepsi_b()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 62, UNIT_DEGREE);
  delay(10000);
  dxl.setGoalPosition(DXL_ID2, 60.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 60.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 216.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 216.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID6, 150, UNIT_DEGREE);
  delay(3000);
}

void two_percent_b()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 86.5, UNIT_DEGREE);
  delay(10000);
  dxl.setGoalPosition(DXL_ID2, 64.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 64.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 216.0, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 216.0, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID6, 150, UNIT_DEGREE);
  delay(3000);
}

void cocacola_b()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 42.5, UNIT_DEGREE);
  delay(10000);
  dxl.setGoalPosition(DXL_ID2, 70, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 70, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 260, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 260, UNIT_DEGREE);
  delay(1000);
  dxl.setGoalPosition(DXL_ID6, 150, UNIT_DEGREE);
  delay(3000);
}

void chilsung_b()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 60.0, UNIT_DEGREE);
  delay(10000);
  dxl.setGoalPosition(DXL_ID2, 70, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 70, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 260, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 260, UNIT_DEGREE);
  delay(2000);
  dxl.setGoalPosition(DXL_ID6, 150, UNIT_DEGREE);
  delay(3000);
}

void tejava_b()
{
  delay(1000);
  dxl.setGoalPosition(DXL_ID1, 82.5, UNIT_DEGREE);
  delay(10000);
  dxl.setGoalPosition(DXL_ID2, 68, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID3, 68, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID4, 260, UNIT_DEGREE);
  dxl.setGoalPosition(DXL_ID5, 260, UNIT_DEGREE);
  delay(1000);
  dxl.setGoalPosition(DXL_ID6, 150, UNIT_DEGREE);
  delay(3000);
}
